# SPDX-License-Identifier: GPL-2.0-or-later
# Copyright (C) 2019-present GeminiTeam ihad.tv (http://ihad.tv)

import xbmcaddon, xbmc
import threading
from os import open as os_open
from os import write as os_write
from os import O_RDWR
from datetime import datetime
import json
import vfddev
import vfdsettings
from vfdutils import *
import struct

vfdchar= (0x5F,
	0x50,
	0x6D,
	0x79,
	0x72,
	0x3B,
	0x3f,
	0x51,
	0x7F,
	0x7B,
	)

addon = xbmcaddon.Addon(id="service.vfd")
vfd_fd = os_open("/dev/vfd", O_RDWR)

class VfdMonitor(xbmc.Monitor):
	def __init__(self):
		super(VfdMonitor, self).__init__()
		self._settingsChangedCallback = None

	def setSettingsChangedCallback(self, callbackObject):
		self._settingsChangedCallback = callbackObject

	def onSettingsChanged(self):
		kodiLog("Enter VfdMonitor.onSettingsChanged")
		if (self._settingsChangedCallback != None):
			self._settingsChangedCallback.onSettingsChanged()

class VfdAddon():
	def __init__(self, monitor):
		self._vfd = vfddev.VfdDev()
		self._states = []
		self._monitor = monitor
		self._monitor.setSettingsChangedCallback(self)
		self._settings = vfdsettings.VfdSettings()
		self._rlock = threading.RLock()
		
	def __getCannel(self):
		# Create request
		request = {
			"jsonrpc": "2.0",
			"method": "Player.GetItem",
			"params": {
				"properties": ["showtitle", "season", "episode", "duration", "file"],
				"playerid": 1
			},
			"id": "VideoGetItem"
		}
		# Make request
		response = xbmc.executeJSONRPC(json.dumps(request))
		xbmc.log("response: " + response)
		try:
			parsed_response = json.loads(response)["result"]["item"]["id"]
		except:
			return -1
		xbmc.log("parsed_response: " + json.dumps(parsed_response, indent=2, sort_keys=True))
		return parsed_response

	def run(self):
		firstLoop = True
		oldCannel = 0
		oldPlayTime = -1
		oldTime = -1
		clock = True
		while not self._monitor.abortRequested():
			if self._monitor.waitForAbort(0.1):
				break
			# loops
			if (firstLoop):
				self.onSettingsChanged()
				firstLoop = False
				
			if(self._settings.isChannelOn() and xbmc.Player().isPlaying() and xbmc.Player().getPlayingFile().startswith("pvr")):
				clock = True
				force = True
				currentCannel = self.__getCannel()
				if currentCannel < 0:
					continue
				if oldCannel != currentCannel:
					oldCannel = currentCannel
					byteCannel = struct.pack(b"bbbbb", vfdchar[(currentCannel%10000)/1000] if (currentCannel%10000)/1000 != 0 else 0x0F, 
										vfdchar[(currentCannel%1000)/100], 
										vfdchar[(currentCannel%100)/10], 
										vfdchar[currentCannel%10], 0)
					os_write(vfd_fd, byteCannel)
					kodiLog("Channel = %d, timeout = %d" %(currentCannel, self._settings.getChannelTimeout()))
					if self._monitor.waitForAbort(self._settings.getChannelTimeout()*1.0):
						break
					
			elif(self._settings.isPlaytimeOn() and xbmc.Player().isPlaying() and not xbmc.Player().getPlayingFile().startswith("pvr")):
				if self._settings.isPlaytimeAsMinutes:
					currentPlayTime = xbmc.Player().getTime()
				else:
					currentPlayTime = xbmc.Player().getTime()/60
				if oldPlayTime != currentPlayTime and currentPlayTime != 0:
					oldPlayTime = currentPlayTime
					clock = False
					bytePlayTime = struct.pack(b"bbbb", currentPlayTime/60, currentPlayTime%60, 0, 1)
					os_write(vfd_fd, bytePlayTime)
					kodiLog("Playtime = %d:%d" %(currentPlayTime/60, currentPlayTime%60))
				elif currentPlayTime == 0:
					clock = True
					force = True
					kodiLog("VfdAddon.run: xbmc.Player().getTime() = 0 set clock")
				else:
					pass
			else:
				clock = True
				
			if clock:
				currentTime = datetime.now().time().hour + datetime.now().time().minute
				if (oldTime != currentTime) or force:
					force = False
					oldTime = currentTime
					byteTime = struct.pack(b"bbbb", datetime.now().time().hour, datetime.now().time().minute, datetime.now().time().second, 1)
					os_write(vfd_fd, byteTime)
					kodiLog("Time = %d:%d" %(datetime.now().time().hour, datetime.now().time().minute))
			
		self.__cleanUp()

	def __cleanUp(self):
		vfd_fd.close()
		self._monitor = None

	def onSettingsChanged(self):
		kodiLog("Enter VfdAddon.onSettingsChanged")
		self._settings.readValues()
		if (self._rlock.acquire()):
			self._vfd.enableDisplay(self._settings.isDisplayOn())
			if (self._settings.isDisplayOn()):
				self._vfd.setBrightness(self._settings.getBrightness())
			self._rlock.release()
			
		kodiLog("isDisplayOn = {0}".format(self._settings.isDisplayOn()))
		kodiLog("isChannelOn = {0}".format(self._settings.isChannelOn()))
		kodiLog("isPlaytimeOn = {0}".format(self._settings.isPlaytimeOn()))
		kodiLog("isPlaytimeAsMinutes = {0}".format(self._settings.isPlaytimeAsMinutes))
		kodiLog("getBrightness = {0}".format(self._settings.getBrightness()))
		kodiLog("getChannelTimeout = {0}".format(self._settings.getChannelTimeout()))

monitor = VfdMonitor()
vfd = VfdAddon(monitor)
kodiLog("Service start.")
vfd.run()
kodiLog("Service stop.")
